#!/usr/bin/env node
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.babelDualPackage = void 0;
var _nodeProcess = require("node:process");
var _nodePerf_hooks = require("node:perf_hooks");
var _nodePath = require("node:path");
var _promises = require("node:fs/promises");
var _nodeFs = require("node:fs");
var _core = require("@babel/core");
var _glob = require("glob");
var _init = require("./init.cjs");
var _transform = require("./transform.cjs");
var _util = require("./util.cjs");
const babelDualPackage = async moduleArgs => {
  const ctx = await (0, _init.init)(moduleArgs, _util.logError);
  if (ctx) {
    const {
      args,
      babelConfig
    } = ctx;
    const {
      targets,
      plugins,
      presets
    } = babelConfig.options;
    const outDir = (0, _nodePath.resolve)((0, _nodePath.relative)((0, _nodeProcess.cwd)(), args.values['out-dir']));
    const cjsOutDir = (0, _nodePath.join)(outDir, args.values['cjs-dir-name']);
    const keepFileExtension = args.values['keep-file-extension'];
    const outFileExtension = args.values['out-file-extension'];
    const noCjsDir = args.values['no-cjs-dir'];
    const sourceMaps = args.values['source-maps'];
    const minified = args.values.minified;
    const extensions = args.values.extensions.split(',', 8).map(ext => ext.trim()).filter(Boolean);
    if (!presets.length) {
      (0, _util.addDefaultPresets)(presets, extensions);
    }
    let numFilesCompiled = 0;
    const esmPresets = (0, _util.getModulePresets)(presets, 'esm');
    const cjsPresets = (0, _util.getModulePresets)(presets, 'cjs');
    const esmPlugins = (0, _util.getEsmPlugins)(plugins);
    const startTime = _nodePerf_hooks.performance.now();
    const build = async (filename, positional) => {
      const {
        code,
        map
      } = await (0, _transform.transform)(filename, {
        targets,
        presets,
        plugins,
        filename,
        minified,
        sourceMaps,
        ast: true,
        sourceType: 'module',
        // Custom options
        esmPresets,
        esmPlugins,
        cjsPresets
      });
      if (code) {
        const extRegex = /(\.\w+)$/i;
        const ext = (0, _nodePath.extname)(filename);
        const relativeFn = positional ? (0, _nodePath.relative)(positional, filename) : (0, _nodePath.basename)(filename);
        const outFile = (0, _nodePath.join)(outDir, relativeFn);
        const outFileCjs = (0, _nodePath.join)(noCjsDir ? outDir : cjsOutDir, relativeFn);
        await (0, _promises.mkdir)((0, _nodePath.dirname)(outFile), {
          recursive: true
        });
        await (0, _promises.mkdir)((0, _nodePath.dirname)(outFileCjs), {
          recursive: true
        });
        await (0, _promises.writeFile)(outFile.replace(extRegex, (0, _util.getOutFileExt)(ext, outFileExtension, keepFileExtension)), code.esm);
        await (0, _promises.writeFile)(outFileCjs.replace(extRegex, (0, _util.getOutFileExt)(ext, outFileExtension, keepFileExtension, 'cjs')), code.cjs);
        if (sourceMaps && map) {
          await (0, _promises.writeFile)(outFile.replace(extRegex, `${(0, _util.getOutFileExt)(ext, outFileExtension, keepFileExtension)}.map`), JSON.stringify(map.esm, null, 2));
          await (0, _promises.writeFile)(outFileCjs.replace(extRegex, `${(0, _util.getOutFileExt)(ext, outFileExtension, keepFileExtension, 'cjs')}.map`), JSON.stringify(map.cjs, null, 2));
        }
      }
    };
    for (const pos of args.positionals) {
      const posPath = (0, _nodePath.resolve)((0, _nodePath.relative)((0, _nodeProcess.cwd)(), pos));
      let stats = null;
      try {
        stats = await (0, _promises.lstat)(posPath);
      } catch {
        // Move to next loop iteration if provided positional results in bogus path
        continue;
      }
      if (stats.isFile()) {
        if (extensions.includes((0, _nodePath.extname)(posPath))) {
          await build(posPath);
          numFilesCompiled++;
        }
      } else {
        const files = (await (0, _util.getFiles)(posPath)).filter(file => extensions.includes((0, _nodePath.extname)(file)));
        for (const filename of files) {
          await build(filename, posPath);
          numFilesCompiled++;
        }
      }
    }
    if (!noCjsDir && !keepFileExtension && !outFileExtension && (0, _nodeFs.existsSync)(outDir) && (0, _nodeFs.existsSync)(cjsOutDir)) {
      /**
       * Copies any .d.ts files from --out-dir to --cjs-dir-name
       * while updating extensions in filenames and import/exports.
       */
      const files = (await (0, _util.getFiles)(outDir)).filter(file => file.endsWith('.d.ts'));
      for (const filename of files) {
        const fileCjs = await (0, _transform.transformDtsExtensions)(filename);
        const filenameCjs = (0, _nodePath.join)(cjsOutDir, (0, _nodePath.basename)(filename));
        await (0, _promises.writeFile)(filenameCjs.replace(/(\.d\.ts)$/, '.d.cts'), fileCjs);
        numFilesCompiled++;
      }
    }
    (0, _util.logResult)(`Successfully compiled ${numFilesCompiled} file${numFilesCompiled === 1 ? '' : 's'} as ESM and CJS in ${Math.round(_nodePerf_hooks.performance.now() - startTime)}ms with Babel ${_core.version}.`);
  }
};
exports.babelDualPackage = babelDualPackage;
const url = new URL(import.meta.url);
const args = url.searchParams.get('args');
const realFileUrlArgv1 = await (0, _util.getRealPathAsFileUrl)(_nodeProcess.argv[1]);
if (parseFloat(_nodeProcess.versions.node) < 16.19) {
  (0, _util.logError)('This script requires a Node version >= 16.19.0');
} else if (import.meta.url === realFileUrlArgv1) {
  await babelDualPackage();
} else if (args) {
  let moduleArgs = null;
  try {
    moduleArgs = JSON.parse(args);
  } catch (err) {
    (0, _util.logError)('Invalid args passed in import query. Not parsable as JSON.');
    (0, _util.logResult)(`Error: ${err.message}`);
  }
  if (moduleArgs) {
    if (moduleArgs.files && Array.isArray(moduleArgs.files)) {
      let positionals = [];

      // Expand any globs and use as positionals, like in a shell
      for (const file of moduleArgs.files) {
        positionals = positionals.concat(await (0, _glob.glob)(file, {
          ignore: 'node_modules/**'
        }));
      }
      delete moduleArgs.files;
      positionals = Object.entries(moduleArgs).concat(positionals);
      moduleArgs = positionals.flat().filter(Boolean);
      await babelDualPackage(moduleArgs);
    } else {
      (0, _util.logError)('No filenames found. Did you forget to pass a "files" key as an array of strings?');
    }
  }
}